/***************************************************************************
 *                                                                         *
 *   PLIS COMPILER                                                         *
 *   =============                                                         *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2007                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef LEXER_H
#define LEXER_H

#include <string>
#include <map>
#include <set>
#include <list>
#include <iostream>

#include "token.h"

class Lexer
{
    static std::map<std::string, Token::Type> m_lang;
    static int m_init;

    std::set<std::string> m_locks;
    std::map<std::string, std::string> m_def;
    std::list<Token> m_list;
    
    bool isNumber(const std::string& token);
    
    Token::Type findType(const std::string& token);

    void expand(std::string& token);

    void next(Token::Context& ctx, std::istream& in, std::string& token);

public:
    typedef std::list<Token>::iterator Iterator;
    typedef std::list<Token>::const_iterator ConstIterator;

    class Exception : public virtual std::exception
    {
	int m_id;
	Token::Context m_ctx; /* Esto tiene un string, hum... */
    public:
	enum {
	    INCLUDE_ERROR,
	    FILE_ERROR,
	    UNEXPECTED_EOF,
	    N_ERROR
	};

	Exception(int id, const Token::Context& ctx) :
	    m_id(id), m_ctx(ctx) {};

	virtual ~Exception() throw() {}
	
	int getID() const {
	    return m_id;
	};
	
	const Token::Context& getContext() const {
	    return m_ctx;
	};
	
	virtual const char* what() const throw() {
	    switch (m_id) {
	    case INCLUDE_ERROR:
		return "You must especify a file path in a string after include.";
	    case FILE_ERROR:
		return "Could not open file.";
	    case UNEXPECTED_EOF:
		return "Unexpected end of file.";
	    default:
		return "Unknown exception.";
	    }
	}
    };
    
    Lexer();
    
    void run(const char* file);

    void finish() {
	m_list.push_back(Token("EOF", Token::END, Token::Context()));
    }
    
    Iterator begin() {
	return m_list.begin();
    }

    ConstIterator begin() const {
	return m_list.begin();
    }

    Iterator end() {
	return m_list.end();
    }

    ConstIterator end() const {
	return m_list.end();
    }
};

#endif /* LEXER_H */
