/*
 * Compilador para el lenguaje PLIS
 *
 * (c) 2007 Juan Pedro Bolívar Puente
 */

#include <iostream>

#include "config.h"
#include "argparser.h"
#include "lexer.h"
#include "compiler.h"

using namespace std;

class OptionHelp : public Option {
public:
    bool parse() {
	cout <<
	    "plisc, plis compiler\n"
	    "(c) 2007 Juan Pedro Bolívar Puente\n"
	    "\n"
	    "Usage:\n"
	    "plisc input_files -o output_file.s [options]\n"
	    "\n"
	    "Options:\n"
	    "  -h, --help           Show help.\n"
	    "  -v, --version        Show version.\n"
	    "  -o, --output <file>  Write output to a file. Else, I will use stdout.\n"
	    "  -s, --start          Write starter function to assemble the output\n"
	    "                       without GCC.\n\n";
	    
	exit(1);
    }
};

class OptionVersion : public Option {
public:
    bool parse() {
	cout << "plisc version " << VERSION << endl;
	exit(1);
    }
};
    
int main(int argc, const char* argv[])
{
    ArgParser args;
    Lexer lex;
    Compiler comp;
    const char* out_file = NULL;
    bool add_start = false;
    
    args.addOption('o', "output", &out_file);
    args.addOption('h', "help", new OptionHelp());
    args.addOption('v', "version", new OptionVersion());
    args.addOption('s', "start", &add_start);
    
    args.parseArgs(argc, argv);

    if (args.numFreeArgs() == 0) {
	cout << "You must give one input file at least. Use -h for help.\n";
	return 1;
    }
    
    try {
	for (ArgParser::Iterator i = args.begin(); i != args.end(); ++i)
	    lex.run(*i);
	lex.finish();
	comp.run(lex, out_file, add_start);
    } catch (Lexer::Exception& e) {
	cerr << e.getContext().file << " ["
	     << e.getContext().line << ':'
	     << e.getContext().col << "] "
	     << e.what() << endl;
	return 1;
    } catch (Compiler::Exception& e) {
	cerr << e.getToken().getContext().file << " ["
	     << e.getToken().getContext().line << ':'
	     << e.getToken().getContext().col << "] "
	     << e.what() << endl;
	return 1;
    } catch (std::exception& e){
	cerr << e.what() << endl;
    }

    return 0;
}
