/***************************************************************************
 *                                                                         *
 *   PLIS COMPILER                                                         *
 *   =============                                                         *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2007                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef TOKEN_H
#define TOKEN_H

#include <string>
#include <map>
#include <list>
#include <stack>
#include <iostream>

class Token
{
public:
    enum Type {
	END,
	
	_LEXERSOPS_START,
	DEF,
	INCLUDE,
	LOCK,
	UNLOCK,
	_LEXEROPS_END,

	FUNC,
	VOID_SENTENCE,
	RETURN,
	ETCETERA,
	END_BLOCK,

	_OPERATORS_START,
	OPERATOR_PLUS,
	OPERATOR_MINUS,
	OPERATOR_MUL,
	OPERATOR_DIV,
	OPERATOR_MOD,
	OPERATOR_PREINC,
	OPERATOR_PREDEC,
	OPERATOR_POSTINC,
	OPERATOR_POSTDEC,
	OPERATOR_SET,
	OPERATOR_BITAND,
	OPERATOR_BITOR,
	OPERATOR_BITXOR,
	OPERATOR_LSHIFT,
	OPERATOR_RSHIFT,
	OPERATOR_SIZE,
	OPERATOR_PARAM,
	OPERATOR_IF,
	OPERATOR_INDEX,

	_CONDITIONALS_START,
	OPERATOR_LT,
	OPERATOR_LTEQ,
	OPERATOR_GT,
	OPERATOR_GTEQ,
	OPERATOR_EQ,
	OPERATOR_NEQ,
	OPERATOR_DIR,
	OPERATOR_DEREF,
	OPERATOR_NOT,
	OPERATOR_AND,
	OPERATOR_OR,
	OPERATOR_XOR,
	_CONDITIONALS_END,
	_OPERATORS_END,

	_STRUCTS_START,
	STRUCT_IF,
	STRUCT_ELSE,
	STRUCT_WHILE,
	STRUCT_REPEAT,
	STRUCT_FOR,
	_STRUCTS_END,

	FLOW_LOOP,
	FLOW_STOP,
	
	_TYPES_START,
	TYPE_CONST,
	TYPE_UNSIGNED,
	TYPE_INT,
	TYPE_SHORT,
	TYPE_BYTE,
	TYPE_PTR,
	TYPE_NONE,
	_TYPES_END,

	NUM_CONST,
	STRING_CONST,
	USER_SYMBOL,
	
	N_TOKEN
    };

    struct Context {
	std::string file;
	int line;
	int col;

	Context() :
	    line(0), col(0) {}

	Context(const std::string& f, int l, int c) :
	    file(f), line(l), col(c) {}

	void update(char c) {
	    if (c == '\n') {
		line++;
		col = 1;
	    } else
		col++;
	}
    };

private:
    std::string m_str;
    Type m_type;
    Context m_ctx;
    
public:

    Token() {}
    
    Token(const std::string& str, Type type, const Context& ctx)
	: m_str(str), m_type(type), m_ctx(ctx) {}

    const Token::Context& getContext() const {
	return m_ctx;
    };
    
    const std::string& getString() const {
	return m_str;
    }

    Type getType() const {
	return m_type;
    }

    bool isOperator() const {
	return m_type > _OPERATORS_START && m_type < _OPERATORS_END;
    }

    bool isConditional() const {
	return m_type > _CONDITIONALS_START && m_type < _CONDITIONALS_END;
    }
    
    bool isType() const {
	return m_type > _TYPES_START && m_type < _TYPES_END;
    }

    bool isStruct() const {
	return m_type > _STRUCTS_START && m_type < _STRUCTS_END;
    }
};


#endif /* TOKEN_H */
